module Handler.CivilServant where

import Import
import UniversityMembersForms (civilServantForm)
import Database.Persist.Sql

import Person (Person (..))
import Employee (Employee(..), EmployeeServant (..))

import UniversityMemberWrapper (qsortByRuleAsc, sortAllByNameReg, joinServants)

getCivilServantsR :: Handler Html
getCivilServantsR = do
    (mmsg) <- getMessage

    cs <- runDB $ selectList [] [Asc CivilServantId]
    sugs <- runDB $ selectList [] [Asc ServantUGStudentId]
    sgs <- runDB $ selectList [] [Asc ServantGStudentId]

    let servants = qsortByRuleAsc sortAllByNameReg $ joinServants cs sugs sgs

    defaultLayout $ do
        setTitle "CivilServants"
        $(widgetFile "civilServant/all")

getOnlyCivilServantsR :: Handler Html
getOnlyCivilServantsR = do
    (mmsg) <- getMessage

    cs <- runDB $ selectList [] [Asc CivilServantId]

    let servants = qsortByRuleAsc sortAllByNameReg $ joinServants cs [] []

    defaultLayout $ do
        setTitle "CivilServants"
        $(widgetFile "civilServant/all")


getNewCivilServantR :: Handler Html
getNewCivilServantR = do
    (mmsg) <- getMessage
    let memberType = "Civil Servant" :: Text
        route = CivilServantsR
    (formWidget, enctype) <- generateFormPost $ civilServantForm Nothing

    defaultLayout $ do
        setTitle "New CivilServant"
        $(widgetFile "member/new")    

postNewCivilServantR :: Handler ()
postNewCivilServantR = do
    ((res, _), _) <- runFormPost $ civilServantForm Nothing

    case res of
        FormSuccess newCivilServant -> do
            mid <- runDB $ getBy $ UniqueTReg (civilServantReg newCivilServant)
            case mid of
                Nothing -> do
                    _ <- runDB $ insert newCivilServant
                    setMessageI MsgMemberSaved
                    redirect $ CivilServantsR
                _ -> do
                    setMessageI MsgDuplicatedMemberError
                    redirect $ NewCivilServantR
        _ -> do
            setMessageI MsgNewMemberError
            redirect $ NewCivilServantR

getEditCivilServantR :: CivilServantId -> Handler Html
getEditCivilServantR tid = do
    (mmsg) <- getMessage
    let memberType = "Civil Servant" :: Text
        route = CivilServantsR
    -- get the civilServant by tid
    civilServant <- runDB $ selectFirst [CivilServantId ==. tid] []
    
    (formWidget, enctype) <- generateFormPost $ civilServantForm $ fmap entityVal civilServant

    defaultLayout $ do
        setTitle "Edit CivilServant"
        $(widgetFile "member/edit")    

postEditCivilServantR :: CivilServantId -> Handler ()
postEditCivilServantR tid = do
    -- get the civilServant by id
    civilServant <- runDB $ get404 tid

    ((res,_), _) <- runFormPost $ civilServantForm $ Just civilServant
    
    case res of
        FormSuccess updatedCivilServant -> do
            _ <- runDB $ replace tid $ updatedCivilServant
            setMessageI MsgMemberSaved
            redirect $ CivilServantsR
        _ -> do
            setMessageI MsgNewMemberError
            redirect $ NewCivilServantR

