module Handler.Graduate where

import Import
import UniversityMembersForms (graduateForm)
import UniversityMemberWrapper (qsortByRuleAsc, sortAllByNameReg, joinGStudents)
import Database.Persist.Sql

import Person (Person (..))
import Student (Student (..), GStudent (..))

getGraduatesR :: Handler Html
getGraduatesR = do
    mmsg <- getMessage
    
    gs <- runDB $ selectList [] [Asc GraduateId]
    tgs <- runDB $ selectList [] [Asc TeacherGStudentId]
    sgs <- runDB $ selectList [] [Asc ServantGStudentId]

    let graduates = qsortByRuleAsc sortAllByNameReg $ joinGStudents gs tgs sgs

    defaultLayout $ do
        setTitle "Graduates"
        $(widgetFile "graduate/all")

getOnlyGraduatesR :: Handler Html
getOnlyGraduatesR = do
    mmsg <- getMessage
    
    gs <- runDB $ selectList [] [Asc GraduateId]

    let graduates = qsortByRuleAsc sortAllByNameReg $ joinGStudents gs [] []

    defaultLayout $ do
        setTitle "Graduates"
        $(widgetFile "graduate/all")

getNewGraduateR :: Handler Html
getNewGraduateR = do
    mmsg <- getMessage
    let memberType = "Graduate Student" :: Text
        route = GraduatesR
    (formWidget, enctype) <- generateFormPost $ graduateForm Nothing

    defaultLayout $ do
        setTitle "New Graduate"
        $(widgetFile "member/new")

postNewGraduateR :: Handler ()
postNewGraduateR = do
    ((res, _), _) <- runFormPost $ graduateForm Nothing

    case res of
        FormSuccess newGraduate -> do
            mid <- runDB $ getBy $ UniqueGSReg (graduateReg newGraduate)
            case mid of
                Nothing -> do
                    _ <- runDB $ insert newGraduate
                    setMessageI MsgMemberSaved
                    redirect $ GraduatesR
                _ -> do
                    setMessageI MsgDuplicatedMemberError
                    redirect $ NewGraduateR
        _ -> do
            setMessageI MsgNewMemberError
            redirect $ NewGraduateR

getEditGraduateR :: GraduateId -> Handler Html
getEditGraduateR gid = do
    (mmsg) <- getMessage
    let memberType = "Graduate Student" :: Text
        route = GraduatesR
    -- get the graduate by id
    graduate <- runDB $ selectFirst [GraduateId ==. gid] []

    (formWidget, enctype) <- generateFormPost $ graduateForm $ fmap entityVal graduate

    defaultLayout $ do
        setTitle "Edit Graduate"
        $(widgetFile "member/edit")    

postEditGraduateR :: GraduateId -> Handler ()
postEditGraduateR gid = do
    -- get the graduate by id
    graduate <- runDB $ get404 gid

    ((res,_), _) <- runFormPost $ graduateForm $ Just graduate

    case res of
        FormSuccess updatedGraduate -> do
            _ <- runDB $ replace gid $ updatedGraduate
            setMessageI MsgMemberSaved
            redirect $ GraduatesR
        _ -> do
            setMessageI MsgNewMemberError
            redirect $ NewGraduateR
