module Handler.Teacher where

import Import
import Database.Persist.Sql
import UniversityMembersForms (teacherForm)
import UniversityMemberWrapper (joinTeachers, sortAllByNameReg, qsortByRuleAsc)
import Employee (showRegime, Employee(..), EmployeeTeacher (..))
import Person (Person (..))

getTeachersR :: Handler Html
getTeachersR = do
    (mmsg) <- getMessage

    ts <- runDB $ selectList [] [Asc TeacherId]
    tugs <- runDB $ selectList [] [Asc TeacherUGStudentId]
    tgs <- runDB $ selectList [] [Asc TeacherGStudentId]

    let teachers = qsortByRuleAsc sortAllByNameReg $ joinTeachers ts tugs tgs
    
    defaultLayout $ do
        setTitle "Teachers"
        $(widgetFile "teacher/all")

getOnlyTeachersR :: Handler Html
getOnlyTeachersR = do
    mmsg <- getMessage

    ts <- runDB $ selectList [] [Asc TeacherId]

    let teachers = qsortByRuleAsc sortAllByNameReg $ joinTeachers ts [] []

    defaultLayout $ do
        setTitle "Teachers"
        $(widgetFile "teacher/all")


getNewTeacherR :: Handler Html
getNewTeacherR = do
    (mmsg) <- getMessage
    let memberType = "Teacher" :: Text
        route = TeachersR
    (formWidget, enctype) <- generateFormPost $ teacherForm Nothing

    defaultLayout $ do
        setTitle "New Teacher"
        $(widgetFile "member/new")

postNewTeacherR :: Handler ()
postNewTeacherR = do
    ((res, _), _) <- runFormPost $ teacherForm Nothing

    case res of
        FormSuccess newTeacher -> do
            mid <- runDB $ getBy $ UniqueTReg (teacherReg newTeacher)
            case mid of
                Nothing -> do
                    _ <- runDB $ insert newTeacher
                    setMessageI MsgMemberSaved
                    redirect $ TeachersR
                _ -> do
                    setMessageI MsgDuplicatedMemberError
                    redirect $ NewTeacherR
        _ -> do
            setMessageI MsgNewMemberError
            redirect $ NewTeacherR

getEditTeacherR :: TeacherId -> Handler Html
getEditTeacherR tid = do
    (mmsg) <- getMessage
    let memberType = "Teacher" :: Text
        route = TeachersR
    -- get the teacher by tid
    teacher <- runDB $ selectFirst [TeacherId ==. tid] []
    
    (formWidget, enctype) <- generateFormPost $ teacherForm $ fmap entityVal teacher

    defaultLayout $ do
        setTitle "Edit Teacher"
        $(widgetFile "member/edit")    

postEditTeacherR :: TeacherId -> Handler ()
postEditTeacherR tid = do
    -- get the teacher by id
    teacher <- runDB $ get404 tid

    ((res,_), _) <- runFormPost $ teacherForm $ Just teacher
    
    case res of
        FormSuccess updatedTeacher -> do
            _ <- runDB $ replace tid $ updatedTeacher
            setMessageI MsgMemberSaved
            redirect $ TeachersR
        _ -> do
            setMessageI MsgNewMemberError
            redirect $ NewTeacherR

