module Handler.Undergraduate where

import Import
import UniversityMembersForms (undergraduateForm)
import UniversityMemberWrapper (qsortByRuleAsc, sortAllByNameReg, joinUGStudents)
import Database.Persist.Sql

import Person (Person (..))
import Student (Student (..), UGStudent (..))

getUndergraduatesR :: Handler Html
getUndergraduatesR = do
    mmsg <- getMessage

    ugs <- runDB $ selectList [] [Asc UndergraduateId]
    tugs <- runDB $ selectList [] [Asc TeacherUGStudentId]
    sugs <- runDB $ selectList [] [Asc ServantUGStudentId]
    
    let undergraduates = qsortByRuleAsc sortAllByNameReg $ joinUGStudents ugs tugs sugs

    defaultLayout $ do
        setTitle "Undergraduates"
        $(widgetFile "undergraduate/all")

getOnlyUndergraduatesR :: Handler Html
getOnlyUndergraduatesR = do
    mmsg <- getMessage

    ugs <- runDB $ selectList [] [Asc UndergraduateId]
    
    let undergraduates = qsortByRuleAsc sortAllByNameReg $ joinUGStudents ugs [] []

    defaultLayout $ do
        setTitle "Undergraduates"
        $(widgetFile "undergraduate/all")

getNewUndergraduateR :: Handler Html
getNewUndergraduateR = do
    mmsg <- getMessage

    let memberType = "Undergraduate Student" :: Text
        route = UndergraduatesR

    (formWidget, enctype) <- generateFormPost $ undergraduateForm Nothing

    defaultLayout $ do
        setTitle "New Undergraduate"
        $(widgetFile "member/new")    

postNewUndergraduateR :: Handler ()
postNewUndergraduateR = do
    ((res, _), _) <- runFormPost $ undergraduateForm Nothing
    
    case res of
        FormSuccess newUndergraduate -> do
            mid <- runDB $ getBy $ UniqueUGSReg (undergraduateReg newUndergraduate)
            case mid of
                Nothing -> do
                    _ <- runDB $ insert newUndergraduate
                    setMessageI MsgMemberSaved
                    redirect $ UndergraduatesR
                _ -> do
                    setMessageI MsgDuplicatedMemberError
                    redirect $ NewUndergraduateR
        _ -> do
            setMessageI MsgNewMemberError
            redirect $ NewUndergraduateR

getEditUndergraduateR :: UndergraduateId -> Handler Html
getEditUndergraduateR ugid = do
    (mmsg) <- getMessage

    let memberType = "Undergraduate Student" :: Text
        route = UndergraduatesR

    -- get the undergraduate by id
    undergraduate <- runDB $ selectFirst [UndergraduateId ==. ugid] []
    
    (formWidget, enctype) <- generateFormPost $ undergraduateForm $ fmap entityVal undergraduate

    defaultLayout $ do
        setTitle "Edit Undergraduate"
        $(widgetFile "member/edit")    

postEditUndergraduateR :: UndergraduateId -> Handler ()
postEditUndergraduateR ugid = do
    -- get the undergraduate by id
    undergraduate <- runDB $ get404 ugid

    ((res,_), _) <- runFormPost $ undergraduateForm $ Just undergraduate
    
    case res of
        FormSuccess updatedUndergraduate -> do
            _ <- runDB $ replace ugid $ updatedUndergraduate
            setMessageI MsgMemberSaved
            redirect $ UndergraduatesR
        _ -> do
            setMessageI MsgNewMemberError
            redirect $ NewUndergraduateR

