module Lib
    (   
            readMaybeInt
        ,   readMaybeInteger
        ,   readMaybeDouble
        ,   splitStrings
        ,   splitByChar
        ,   readMaybeDay
        ,   formatDate
        ,   readMaybeUndergraduate
        ,   readMaybeGraduate
        ,   readMaybeTeacher
        ,   readMaybeTeacherUGStudent
        ,   readMaybeTeacherGStudent
        ,   readMaybeCivilServant
        ,   readMaybeServantUGS
        ,   readMaybeServantGS
    ) where

import Import
import Data.List ((!!))
import Data.Time.Calendar (fromGregorianValid)
import Data.Time.Calendar.Julian (toJulian)
import Text.Read (readMaybe)
import Data.Text (pack)


import Person (readMaybeGenre)
import Employee (readMaybeRegime, readMaybeOutsourcing)

-- convert String to Maybe Int
readMaybeInt :: String -> Maybe Int
readMaybeInt s = readMaybe s :: Maybe Int

-- convert String to Maybe Integer
readMaybeInteger :: String -> Maybe Integer
readMaybeInteger s = readMaybe s :: Maybe Integer

-- convert String to Maybe Double
readMaybeDouble :: String -> Maybe Double
readMaybeDouble s = readMaybe s :: Maybe Double   

-- split strings
splitStrings :: [String] -> [String] -> [[String]]
splitStrings [] [] = []
splitStrings [] tmp = [tmp]
splitStrings (l:ls) tmp =
    if length l < 1
        then
            tmp : splitStrings ls []
        else
            splitStrings ls (tmp ++ [l])
            

-- readMaybeDay
readMaybeDay :: String -> Maybe Day
readMaybeDay "" = Nothing
readMaybeDay day = 
    case (readMaybeInt d, readMaybeInt m, readMaybeInteger y) of
        (Just d', Just m', Just y') -> fromGregorianValid y' m' d'
        _ -> Nothing
    where
        ss = splitByChar '/' day ""
        (d, m, y) = 
            if 3 == length ss
                then
                    (ss!!0, ss!!1, ss!!2)
                else
                    ("", "", "")

formatDate :: Day -> String
formatDate day =
    show d ++ "/" ++ show m ++ "/" ++ show y
    where
        (y, m, d) = toJulian day

splitByChar :: Char -> String -> String -> [String]
splitByChar _ "" tmp = [tmp]
splitByChar c (s:ss) tmp =
    if s == c
        then
            tmp : splitByChar c ss ""
        else
            splitByChar c ss (tmp ++ [s])

---
readMaybeGraduate :: [String] -> Maybe Graduate
readMaybeGraduate [sName, sAge, sGenre, sReg, sAdm, sCourse, sGpa, sGrad] =
    case (readMaybeInt sAge, readMaybeGenre sGenre, readMaybeInt sReg, readMaybeDay sAdm, readMaybeDouble sGpa) of
        (Just age', Just genre', Just reg', Just adm', Just gpa') -> Just $ Graduate (Data.Text.pack sName) age' genre' reg' adm' (Data.Text.pack sCourse) gpa' (Data.Text.pack sGrad)
        _ -> Nothing
readMaybeGraduate _ = Nothing

--
readMaybeUndergraduate :: [String] -> Maybe Undergraduate
readMaybeUndergraduate [sName, sAge, sGenre, sReg, sAdm, sCourse, sGpa, sScore] =
    case (readMaybeInt sAge, readMaybeGenre sGenre, readMaybeInt sReg, readMaybeDay sAdm, readMaybeDouble sGpa, readMaybeInt sScore) of
        (Just age', Just genre', Just reg', Just adm', Just gpa', Just score') -> Just $ Undergraduate (Data.Text.pack sName) age' genre' reg' adm' (Data.Text.pack sCourse) gpa' score'
        _ -> Nothing
readMaybeUndergraduate _ = Nothing

---
readMaybeTeacher :: [String] -> Maybe Teacher
readMaybeTeacher [tName, tAge, tGenre, tReg, tAdm, tSal, tRegime] =
    case (readMaybeInt tAge, readMaybeGenre tGenre, readMaybeInt tReg, readMaybeDay tAdm, readMaybeDouble tSal, readMaybeRegime tRegime) of
        (Just age', Just genre', Just reg', Just adm', Just sal', Just regime') -> Just $ Teacher (Data.Text.pack tName) age' genre' reg' adm' sal' regime'
        _ -> Nothing
readMaybeTeacher _ = Nothing

---
-- read a [String] and try to convert to TeacherUGStudent
readMaybeTeacherUGStudent :: [String] -> Maybe TeacherUGStudent
readMaybeTeacherUGStudent   [ tNam, tAge, tGen, tRgE, tAdmE, tSal, tRegime, tRgS, tAdmS, tCourse, tGpa, tSco ] =
    case (  readMaybeInt tAge,
            readMaybeGenre tGen,
            readMaybeInt tRgE,
            readMaybeDay tAdmE,
            readMaybeDouble tSal,
            readMaybeRegime tRegime,
            readMaybeInt tRgS,
            readMaybeDay tAdmS,
            readMaybeDouble tGpa,
            readMaybeInt tSco ) of
        (
            Just age',
            Just genre',
            Just regE',
            Just admE',
            Just sal',
            Just regime',
            Just regS',
            Just admS',
            Just gpa',
            Just score') -> Just $ TeacherUGStudent (Data.Text.pack tNam) age' genre' regE' admE' sal' regime' regS' admS' (Data.Text.pack tCourse) gpa' score' 
        _ -> Nothing
readMaybeTeacherUGStudent _ = Nothing

---
-- read a [String] and try to convert to TeacherGStudent
readMaybeTeacherGStudent :: [String] -> Maybe TeacherGStudent
readMaybeTeacherGStudent   [ tNam, tAge, tGen, tRgE, tAdmE, tSal, tRegime, tRgS, tAdmS, tCourse, tGpa, tGrad ] =
    case (  readMaybeInt tAge,
            readMaybeGenre tGen,
            readMaybeInt tRgE,
            readMaybeDay tAdmE,
            readMaybeDouble tSal,
            readMaybeRegime tRegime,
            readMaybeInt tRgS,
            readMaybeDay tAdmS,
            readMaybeDouble tGpa
        ) of
        (
            Just age',
            Just genre',
            Just regE',
            Just admE',
            Just sal',
            Just regime',
            Just regS',
            Just admS',
            Just gpa' ) -> Just $ TeacherGStudent (Data.Text.pack tNam) age' genre' regE' admE' sal' regime' regS' admS' (Data.Text.pack tCourse) gpa' (Data.Text.pack tGrad)
        _ -> Nothing
readMaybeTeacherGStudent _ = Nothing

---
readMaybeCivilServant :: [String] -> Maybe CivilServant
readMaybeCivilServant [csName, csAge, csGenre, csReg, csAdm, csSal, csOut] =
    case (readMaybeInt csAge, readMaybeGenre csGenre, readMaybeInt csReg, readMaybeDay csAdm, readMaybeDouble csSal, readMaybeOutsourcing csOut) of
        (Just age', Just genre', Just reg', Just adm', Just sal', Just outsourcing') -> Just $ CivilServant (Data.Text.pack csName) age' genre' reg' adm' sal' outsourcing'
        _ -> Nothing
readMaybeCivilServant _ = Nothing

---
-- read a [String] and try to convert to ServantUGStudent
readMaybeServantGS :: [String] -> Maybe ServantGStudent
readMaybeServantGS [csNam, csAge, csGen, csRegE, csAdmE, csSal, csOut, csRegS, csAdmS, csCour, csGpa, csGrad] =
    case (  readMaybeInt csAge,
            readMaybeGenre csGen,
            readMaybeInt csRegE,
            readMaybeDay csAdmE,
            readMaybeDouble csSal,
            readMaybeOutsourcing csOut,
            readMaybeInt csRegS,
            readMaybeDay csAdmS,
            readMaybeDouble csGpa ) of
        (   
            Just age',
            Just genre',
            Just regE',
            Just admE',
            Just sal',
            Just out',
            Just regS',
            Just admS',
            Just gpa') -> Just $ ServantGStudent (Data.Text.pack csNam) age' genre' regE' admE' sal' out' regS' admS' (Data.Text.pack csCour) gpa' (Data.Text.pack csGrad)
        _ -> Nothing
readMaybeServantGS _ = Nothing

--
-- read a [String] and try to convert to ServantUGStudent
readMaybeServantUGS :: [String] -> Maybe ServantUGStudent
readMaybeServantUGS [csNam, csAge, csGen, csRegE, csAdmE, csSal, csOut, csRegS, csAdmS, csCour, csGpa, csSco] =
    case (  readMaybeInt csAge,
            readMaybeGenre csGen,
            readMaybeInt csRegE,
            readMaybeDay csAdmE,
            readMaybeDouble csSal,
            readMaybeOutsourcing csOut,
            readMaybeInt csRegS,
            readMaybeDay csAdmS,
            readMaybeDouble csGpa,
            readMaybeInt csSco ) of
        (   
            Just age',
            Just genre',
            Just regE',
            Just admE',
            Just sal',
            Just out',
            Just regS',
            Just admS',
            Just gpa',
            Just sco') -> Just $ ServantUGStudent (Data.Text.pack csNam) age' genre' regE' admE' sal' out' regS' admS' (Data.Text.pack csCour) gpa' sco'
        _ -> Nothing
readMaybeServantUGS _ = Nothing
