module UniversityMemberWrapper (
        selectMembers
    ,   isTeacher
    ,   isTeacher20
    ,   isTeacher40
    ,   isTeacherED
    ,   isStudent
    ,   isUndergraduate
    ,   isGraduate
    ,   isEmployee
    ,   isCivilServant
    ,   hasTwoRegistrations
    ,   getMaxReg
    ,   sortAllByNameReg
    ,   sortStudentByGpaReg
    ,   sortTeacherByReg
    ,   sortEmployeeBySalReg
    ,   sortGraduateByAgeReg
    ,   qsortByRule
    ,   qsortByRuleAsc
    ,   qsortByRuleDesc
    ,   parseMembers
    ,   buildMember
    ,   joinUGStudents
    ,   joinGStudents
    ,   joinTeachers
    ,   joinServants
    ) where

import Import       
import Database.Persist.Sql (fromSqlKey)
import Lib (    
                readMaybeUndergraduate
            ,   readMaybeGraduate
            ,   readMaybeTeacher
            ,   readMaybeTeacherUGStudent
            ,   readMaybeTeacherGStudent
            ,   readMaybeCivilServant
            ,   readMaybeServantUGS
                ,   readMaybeServantGS
            )

import Person (Person (..))
import Employee (Regime (..), Employee(..))
import Student (Student (..))

---------------------------------------------------------------------------
---------------------------------------------------------------------------
-- get all Students from an UniversityMember list
selectMembers :: (UniversityMember -> Bool) -> [UniversityMember] -> [UniversityMember]
selectMembers _ [] = []
selectMembers select ms = [x | x <- ms, select x]

---------------------------------------------------------------------------
---------------------------------------------------------------------------
-- the selection methods
isTeacher :: UniversityMember -> Bool
isTeacher um =
    case um of
        TMember _ -> True
        TUGSMember _ -> True
        TGSMember _ -> True
        _ -> False

-- the selection methods
isTeacher40 :: UniversityMember -> Bool
isTeacher40 um =
    case um of
        TMember t -> R40 == teacherRegime t
        TUGSMember t -> R40 == teacherUGStudentRegime t
        TGSMember t -> R40 == teacherGStudentRegime t
        _ -> False

-- the selection methods
isTeacher20 :: UniversityMember -> Bool
isTeacher20 um =
    case um of
        TMember t -> R20 == teacherRegime t
        TUGSMember t -> R20 == teacherUGStudentRegime t
        TGSMember t -> R20 == teacherGStudentRegime t
        _ -> False

-- the selection methods
isTeacherED :: UniversityMember -> Bool
isTeacherED um =
    case um of
        TMember t -> RED == teacherRegime t
        TUGSMember t -> RED == teacherUGStudentRegime t
        TGSMember t -> RED == teacherGStudentRegime t
        _ -> False
---------------------------------------------------------------------------
---------------------------------------------------------------------------
-- the students selection methods
-- verify if an UniversityMember is a Student
isStudent :: UniversityMember -> Bool
isStudent um =
    case um of
        TMember _ -> False
        CSMember _ -> False
        _ -> True

-- verify if an UniversityMember is an Undergraduate Student
isUndergraduate :: UniversityMember -> Bool
isUndergraduate um =
    case um of
        UGSMember _ -> True
        TUGSMember _ -> True
        SUGSMember _ -> True
        _ -> False

-- verify if an UniversityMember is an Graduate Student
isGraduate :: UniversityMember -> Bool
isGraduate um =
    case um of
        GSMember _ -> True
        TGSMember _ -> True
        SGSMember _ -> True
        _ -> False

-- verify if an UniversityMember has 2 registrations
hasTwoRegistrations :: UniversityMember -> Bool
hasTwoRegistrations um =
    case um of
        UGSMember _ -> False
        GSMember _ -> False
        TMember _ -> False
        CSMember _ -> False
        _ -> True

---------------------------------------------------------------------------
---------------------------------------------------------------------------
-- the Employees selection methods
-- verify if an UniversityMember is an Employee
isEmployee :: UniversityMember -> Bool
isEmployee um =
    case um of
        UGSMember _ -> False
        GSMember _ -> False
        _ -> True
---------------------------------------------------------------------------
---------------------------------------------------------------------------
-- the Civil Servants selection methods
-- verify if an UniversityMember is a Civil Servant
isCivilServant :: UniversityMember -> Bool
isCivilServant um =
    case um of
        CSMember _ -> True
        SUGSMember _ -> True
        SGSMember _ -> True
        _ -> False

------------------------------------------------------------------------------
------------------------------------------------------------------------------
getMaxReg :: UniversityMember -> Int
getMaxReg um =
    case um of
        UGSMember m -> s_reg m
        GSMember m -> s_reg m
        TMember m -> e_reg m
        TUGSMember m -> max (e_reg m) (s_reg m)
        TGSMember m -> max (e_reg m) (s_reg m)
        CSMember m -> e_reg m
        SUGSMember m -> max (e_reg m) (s_reg m)
        SGSMember m -> max (e_reg m) (s_reg m)

------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- a rule to ord UniversityMember by name and registration
sortAllByNameReg :: UniversityMember -> UniversityMember -> Bool
sortAllByNameReg a b = name a < name b || name a == name b && getMaxReg a < getMaxReg b

-- a rule to sort Teachers with 40 h regime by employee registration
sortTeacherByReg :: UniversityMember -> UniversityMember -> Bool
sortTeacherByReg a b = e_reg a < e_reg b

-- a rule to sort Graduate Students by age
sortGraduateByAgeReg :: UniversityMember -> UniversityMember -> Bool
sortGraduateByAgeReg a b = age a < age b || age a == age b && s_reg a < s_reg b 

-- a rule to sort all Students by s_gpa and registration
sortStudentByGpaReg :: UniversityMember -> UniversityMember -> Bool
sortStudentByGpaReg a b = s_gpa a < s_gpa b || s_gpa a == s_gpa b && s_reg a < s_reg b

-- a rule to sort all Employees by salary and registration
sortEmployeeBySalReg :: UniversityMember -> UniversityMember -> Bool
sortEmployeeBySalReg a b = salary a < salary b || salary a == salary b && e_reg a < e_reg b

---------------------------------------------------------------------------
---------------------------------------------------------------------------
qsortByRuleAsc :: (UniversityMember -> UniversityMember -> Bool) -> [(Int64, UniversityMember)] -> [(Int64, UniversityMember)]
qsortByRuleAsc _ [] = []
qsortByRuleAsc rule ((k,w):ms) =
    qsortByRuleAsc rule leftSide ++ [(k,w)] ++ qsortByRuleAsc rule rightSide
    where
        leftSide =  [(i,x) | (i,x) <- ms, rule x w]
        rightSide = [(j,y) | (j,y) <- ms, not $ rule y w]

qsortByRuleDesc :: (UniversityMember -> UniversityMember -> Bool) -> [(Int64, UniversityMember)] -> [(Int64, UniversityMember)]
qsortByRuleDesc _ [] = []
qsortByRuleDesc rule ((k,w):ms) =
    qsortByRuleDesc rule leftSide ++ [(k,w)] ++ qsortByRuleDesc rule rightSide
    where
        leftSide =  [(i,x) | (i,x) <- ms, not $ rule x w]
        rightSide = [(j,y) | (j,y) <- ms, rule y w]

qsortByRule :: Bool -> (UniversityMember -> UniversityMember -> Bool) -> [(Int64, UniversityMember)] -> [(Int64, UniversityMember)]
qsortByRule True rule ms = qsortByRuleAsc rule ms
qsortByRule False rule ms = qsortByRuleDesc rule ms

---------------------------------------------------------------------------
---------------------------------------------------------------------------
parseMembers :: [[String]] -> [UniversityMember]
parseMembers [] = []
parseMembers (m:ms) = 
    case buildMember m of
        Just um -> um : parseMembers ms
        Nothing -> parseMembers ms

---------------------------------------------------------------------------
---------------------------------------------------------------------------

buildMember :: [String] -> Maybe UniversityMember
buildMember [] = Nothing
buildMember (mt:md) = 
    case mt of
        "grad" ->       case readMaybeUndergraduate md of
                            Just m -> Just $ UGSMember m
                            _ -> Nothing
        "pos" ->        case readMaybeGraduate md of
                            Just m -> Just $ GSMember m
                            _ -> Nothing
        "prof" ->       case readMaybeTeacher md of
                            Just m -> Just $ TMember m
                            _ -> Nothing
        "prof_grad" ->  case readMaybeTeacherUGStudent md of
                            Just m -> Just $ TUGSMember m
                            _ -> Nothing
        "prof_pos" ->   case readMaybeTeacherGStudent md of
                            Just m -> Just $ TGSMember m
                            _ -> Nothing
        "serv" ->       case readMaybeCivilServant md of
                            Just m -> Just $ CSMember m
                            _ -> Nothing
        "serv_grad" ->  case readMaybeServantUGS md of
                            Just m -> Just $ SUGSMember m
                            _ -> Nothing
        "serv_pos" ->   case readMaybeServantGS md of
                            Just m -> Just $ SGSMember m
                            _ -> Nothing
        _ -> Nothing
---------------------------------------------------------------------------
---------------------------------------------------------------------------
---------------------------------------------------------------------------
joinUGStudents :: [Entity Undergraduate] -> [Entity TeacherUGStudent] -> [Entity ServantUGStudent] -> [(Int64, UniversityMember)] 
joinUGStudents ugs tugs sugs = a ++ b ++ c
    where
        a = [(fromSqlKey sid, UGSMember student)  | (Entity sid student) <- ugs]
        b = [(fromSqlKey tid, TUGSMember teacher) | (Entity tid teacher ) <- tugs]
        c = [(fromSqlKey cid, SUGSMember servant) | (Entity cid servant) <- sugs]

---------------------------------------------------------------------------
---------------------------------------------------------------------------
joinGStudents :: [Entity Graduate] -> [Entity TeacherGStudent] -> [Entity ServantGStudent] -> [(Int64, UniversityMember)] 
joinGStudents gs tgs sgs = a ++ b ++ c
    where
        a = [(fromSqlKey sid, GSMember student)  | (Entity sid student) <- gs]
        b = [(fromSqlKey tid, TGSMember teacher) | (Entity tid teacher ) <- tgs]
        c = [(fromSqlKey cid, SGSMember servant) | (Entity cid servant) <- sgs]

---------------------------------------------------------------------------
---------------------------------------------------------------------------
joinTeachers :: [Entity Teacher] -> [Entity TeacherUGStudent] -> [Entity TeacherGStudent] -> [(Int64, UniversityMember)] 
joinTeachers ts tugs tgs = a ++ b ++ c
    where
        a = [(fromSqlKey tid, TMember ts')  | (Entity tid ts') <- ts]
        b = [(fromSqlKey tugid, TUGSMember tugs') | (Entity tugid tugs') <- tugs]
        c = [(fromSqlKey tgid, TGSMember tgs') | (Entity tgid tgs') <- tgs]
---------------------------------------------------------------------------
---------------------------------------------------------------------------
joinServants :: [Entity CivilServant] -> [Entity ServantUGStudent] -> [Entity ServantGStudent] -> [(Int64, UniversityMember)] 
joinServants cs sugs sgs = a ++ b ++ c
    where
        a = [(fromSqlKey cid, CSMember cs')  | (Entity cid cs') <- cs]
        b = [(fromSqlKey sugid, SUGSMember sugs') | (Entity sugid sugs') <- sugs]
        c = [(fromSqlKey sgid, SGSMember sgs') | (Entity sgid sgs') <- sgs]


