module UniversityMembersForms 
    (
        teacherForm
    ,   civilServantForm
    ,   undergraduateForm
    ,   graduateForm
    ,   teacherUGStudentForm
    ,   teacherGStudentForm
    ,   servantUGStudentForm
    ,   servantGStudentForm
    )
    where

import Import
import Yesod.Form.Bootstrap3 ( BootstrapFormLayout (..), BootstrapGridOptions (..), renderBootstrap3, withPlaceholder, bfs)

import Person (Genre (..))
import Employee (Regime (..), Outsourcing (..))

largeLayout :: BootstrapFormLayout
largeLayout = BootstrapHorizontalForm (ColSm 0) (ColSm 3) (ColSm 0) (ColSm 6)

genres ::  [(Text, Genre)]
genres = [("Female", Female), ("Male", Male)]

regimes :: [(Text, Regime)]
regimes = [("DE", RED), ("20", R20), ("40", R40)] 

outsource :: [(Text, Outsourcing)]
outsource = [("V", V), ("F", F)]


regField = areq intField (bfs ("Registration" :: Text)) Nothing

----------------------------------------------------------------------------------------------------------
-- THE TEACHERS FORM
----------------------------------------------------------------------------------------------------------
teacherAForm :: Maybe Teacher -> AForm Handler Teacher
teacherAForm mt = Teacher
    <$> areq textField (withPlaceholder "full name" $ bfs ("Name" :: Text)) (teacherName <$> mt)
    <*> areq intField (bfs ("Age" :: Text)) (teacherAge <$> mt)
    <*> areq (selectFieldList genres) (bfs ("Genre"::Text)) (teacherGenre <$> mt)
    <*> maybeTRegField mt
    <*> areq dayField (bfs ("Admission date" :: Text )) (teacherAdm <$> mt)
    <*> areq doubleField (bfs ("Salary"::Text)) (teacherSal <$> mt)
    <*> areq (selectFieldList regimes) (bfs ("Genre"::Text)) (teacherRegime <$> mt)

    where
        maybeTRegField Nothing = regField
        maybeTRegField (Just t) = pure $ teacherReg t

teacherForm :: Maybe Teacher -> Html -> MForm Handler (FormResult Teacher, Widget)
teacherForm mt = renderBootstrap3 largeLayout $ teacherAForm mt

----------------------------------------------------------------------------------------------------------
-- THE CIVIL SERVANTS FORM
----------------------------------------------------------------------------------------------------------
civilServantAForm :: Maybe CivilServant -> AForm Handler CivilServant
civilServantAForm mcs = CivilServant
    <$> areq textField (withPlaceholder "full name" $ bfs ("Name" :: Text)) (civilServantName <$> mcs)
    <*> areq intField (bfs ("Age" :: Text)) (civilServantAge <$> mcs)
    <*> areq (selectFieldList genres) (bfs ("Genre"::Text)) (civilServantGenre <$> mcs)
    <*> maybeCSRegField mcs
    <*> areq dayField (bfs ("Admission date" :: Text )) (civilServantAdm <$> mcs)
    <*> areq doubleField (bfs ("Salary"::Text)) (civilServantSal <$> mcs)
    <*> areq (selectFieldList outsource) (bfs ("Outsourced"::Text)) (civilServantOuts <$> mcs)

    where
        maybeCSRegField Nothing = regField
        maybeCSRegField (Just t) = pure $ civilServantReg t

civilServantForm :: Maybe CivilServant -> Html -> MForm Handler (FormResult CivilServant, Widget)
civilServantForm mcs = renderBootstrap3 largeLayout $ civilServantAForm mcs

----------------------------------------------------------------------------------------------------------
-- THE UNDERGRADUATE FORM
----------------------------------------------------------------------------------------------------------
undergraduateAForm :: Maybe Undergraduate -> AForm Handler Undergraduate
undergraduateAForm mugs = Undergraduate
    <$> areq textField (withPlaceholder "full name" $ bfs ("Name" :: Text)) (undergraduateName <$> mugs)
    <*> areq intField (bfs ("Age" :: Text)) (undergraduateAge <$> mugs)
    <*> areq (selectFieldList genres) (bfs ("Genre"::Text)) (undergraduateGenre <$> mugs)
    <*> maybeUGSRegField mugs
    <*> areq dayField (bfs ("Admission date" :: Text )) (undergraduateAdm <$> mugs)
    <*> areq textField (bfs ("Course"::Text)) (undergraduateCourse <$> mugs)
    <*> areq doubleField (bfs ("GPA"::Text)) (undergraduateGpa <$> mugs)
    <*> areq intField (bfs ("Score"::Text)) (undergraduateScore <$> mugs)

    where
        maybeUGSRegField Nothing = regField
        maybeUGSRegField (Just ugs) = pure $ undergraduateReg ugs

undergraduateForm :: Maybe Undergraduate -> Html -> MForm Handler (FormResult Undergraduate, Widget)
undergraduateForm mugs = renderBootstrap3 largeLayout $ undergraduateAForm mugs

----------------------------------------------------------------------------------------------------------
-- THE GRADUATE FORM
----------------------------------------------------------------------------------------------------------
graduateAForm :: Maybe Graduate -> AForm Handler Graduate
graduateAForm mgs = Graduate
    <$> areq textField (withPlaceholder "full name" $ bfs ("Name" :: Text)) (graduateName <$> mgs)
    <*> areq intField (bfs ("Age" :: Text)) (graduateAge <$> mgs)
    <*> areq (selectFieldList genres) (bfs ("Genre"::Text)) (graduateGenre <$> mgs)
    <*> maybeGSRegField mgs
    <*> areq dayField (bfs ("Admission date" :: Text )) (graduateAdm <$> mgs)
    <*> areq textField (bfs ("Course"::Text)) (graduateCourse <$> mgs)
    <*> areq doubleField (bfs ("GPA"::Text)) (graduateGpa <$> mgs)
    <*> areq textField (bfs ("Score"::Text)) (graduateGrad <$> mgs)

    where
        maybeGSRegField Nothing = regField
        maybeGSRegField (Just gs) = pure $ graduateReg gs

graduateForm :: Maybe Graduate -> Html -> MForm Handler (FormResult Graduate, Widget)
graduateForm mgs = renderBootstrap3 largeLayout $ graduateAForm mgs

----------------------------------------------------------------------------------------------------------
-- THE TEACHERUGSTUDENT FORM
----------------------------------------------------------------------------------------------------------
teacherUGStudentAForm :: Maybe TeacherUGStudent -> AForm Handler TeacherUGStudent
teacherUGStudentAForm mtugs = TeacherUGStudent
    <$> areq textField (withPlaceholder "full name" $ bfs ("Name" :: Text)) (teacherUGStudentName <$> mtugs)
    <*> areq intField (bfs ("Age" :: Text)) (teacherUGStudentAge <$> mtugs)
    <*> areq (selectFieldList genres) (bfs ("Genre"::Text)) (teacherUGStudentGenre <$> mtugs)
    <*> maybeTUGSRegEField mtugs
    <*> areq dayField (bfs ("Employee admission date" :: Text )) (teacherUGStudentAdmE <$> mtugs)
    <*> areq doubleField (bfs ("Salary" :: Text)) (teacherUGStudentSal <$> mtugs)
    <*> areq (selectFieldList regimes) (bfs ("Regime" :: Text)) (teacherUGStudentRegime <$> mtugs)
    <*> maybeTUGSRegSField mtugs
    <*> areq dayField (bfs ("Student admission date" :: Text )) (teacherUGStudentAdmS <$> mtugs)
    <*> areq textField (bfs ("Course"::Text)) (teacherUGStudentCourse <$> mtugs)
    <*> areq doubleField (bfs ("GPA"::Text)) (teacherUGStudentGpa <$> mtugs)
    <*> areq intField (bfs ("Score"::Text)) (teacherUGStudentScore <$> mtugs)

    where
        maybeTUGSRegEField Nothing = regField
        maybeTUGSRegEField (Just tugs) = pure $ teacherUGStudentRegE tugs

        maybeTUGSRegSField Nothing = regField
        maybeTUGSRegSField (Just tugs) = pure $ teacherUGStudentRegS tugs

teacherUGStudentForm :: Maybe TeacherUGStudent -> Html -> MForm Handler (FormResult TeacherUGStudent, Widget)
teacherUGStudentForm mtugs = renderBootstrap3 largeLayout $ teacherUGStudentAForm mtugs

----------------------------------------------------------------------------------------------------------
-- THE TEACHERGSTUDENT FORM
----------------------------------------------------------------------------------------------------------
teacherGStudentAForm :: Maybe TeacherGStudent -> AForm Handler TeacherGStudent
teacherGStudentAForm mtgs = TeacherGStudent
    <$> areq textField (withPlaceholder "full name" $ bfs ("Name" :: Text)) (teacherGStudentName <$> mtgs)
    <*> areq intField (bfs ("Age" :: Text)) (teacherGStudentAge <$> mtgs)
    <*> areq (selectFieldList genres) (bfs ("Genre"::Text)) (teacherGStudentGenre <$> mtgs)
    <*> maybeTGSRegEField mtgs
    <*> areq dayField (bfs ("Employee admission date" :: Text )) (teacherGStudentAdmE <$> mtgs)
    <*> areq doubleField (bfs ("Salary" :: Text)) (teacherGStudentSal <$> mtgs)
    <*> areq (selectFieldList regimes) (bfs ("Regime" :: Text)) (teacherGStudentRegime <$> mtgs)
    <*> maybeTGSRegSField mtgs
    <*> areq dayField (bfs ("Student admission date" :: Text )) (teacherGStudentAdmS <$> mtgs)
    <*> areq textField (bfs ("Course"::Text)) (teacherGStudentCourse <$> mtgs)
    <*> areq doubleField (bfs ("GPA"::Text)) (teacherGStudentGpa <$> mtgs)
    <*> areq textField (bfs ("Graduation"::Text)) (teacherGStudentGrad <$> mtgs)

    where
        maybeTGSRegEField Nothing = regField
        maybeTGSRegEField (Just tgs) = pure $ teacherGStudentRegE tgs

        maybeTGSRegSField Nothing = regField
        maybeTGSRegSField (Just tgs) = pure $ teacherGStudentRegS tgs

teacherGStudentForm :: Maybe TeacherGStudent -> Html -> MForm Handler (FormResult TeacherGStudent, Widget)
teacherGStudentForm mtgs = renderBootstrap3 largeLayout $ teacherGStudentAForm mtgs

----------------------------------------------------------------------------------------------------------
-- THE SERVANTUGSTUDENT FORM
----------------------------------------------------------------------------------------------------------
servantUGStudentAForm :: Maybe ServantUGStudent -> AForm Handler ServantUGStudent
servantUGStudentAForm msugs = ServantUGStudent
    <$> areq textField (withPlaceholder "full name" $ bfs ("Name" :: Text)) (servantUGStudentName <$> msugs)
    <*> areq intField (bfs ("Age" :: Text)) (servantUGStudentAge <$> msugs)
    <*> areq (selectFieldList genres) (bfs ("Genre"::Text)) (servantUGStudentGenre <$> msugs)
    <*> maybeSUGSRegEField msugs
    <*> areq dayField (bfs ("Employee admission date" :: Text )) (servantUGStudentAdmE <$> msugs)
    <*> areq doubleField (bfs ("Salary" :: Text)) (servantUGStudentSal <$> msugs)
    <*> areq (selectFieldList outsource) (bfs ("Outsourcing" :: Text)) (servantUGStudentOuts <$> msugs)
    <*> maybeSUGSRegSField msugs
    <*> areq dayField (bfs ("Student admission date" :: Text )) (servantUGStudentAdmS <$> msugs)
    <*> areq textField (bfs ("Course"::Text)) (servantUGStudentCourse <$> msugs)
    <*> areq doubleField (bfs ("GPA"::Text)) (servantUGStudentGpa <$> msugs)
    <*> areq intField (bfs ("Score"::Text)) (servantUGStudentScore <$> msugs)

    where
        maybeSUGSRegEField Nothing = regField
        maybeSUGSRegEField (Just tgs) = pure $ servantUGStudentRegE tgs

        maybeSUGSRegSField Nothing = regField
        maybeSUGSRegSField (Just tgs) = pure $ servantUGStudentRegS tgs

servantUGStudentForm :: Maybe ServantUGStudent -> Html -> MForm Handler (FormResult ServantUGStudent, Widget)
servantUGStudentForm msugs = renderBootstrap3 largeLayout $ servantUGStudentAForm msugs

----------------------------------------------------------------------------------------------------------
-- THE SERVANTGSTUDENT FORM
----------------------------------------------------------------------------------------------------------
servantGStudentAForm :: Maybe ServantGStudent -> AForm Handler ServantGStudent
servantGStudentAForm msgs = ServantGStudent
    <$> areq textField (withPlaceholder "full name" $ bfs ("Name" :: Text)) (servantGStudentName <$> msgs)
    <*> areq intField (bfs ("Age" :: Text)) (servantGStudentAge <$> msgs)
    <*> areq (selectFieldList genres) (bfs ("Genre"::Text)) (servantGStudentGenre <$> msgs)
    <*> maybeSGSRegEField msgs
    <*> areq dayField (bfs ("Employee admission date" :: Text )) (servantGStudentAdmE <$> msgs)
    <*> areq doubleField (bfs ("Salary" :: Text)) (servantGStudentSal <$> msgs)
    <*> areq (selectFieldList outsource) (bfs ("Outsourcing" :: Text)) (servantGStudentOuts <$> msgs)
    <*> maybeSGSRegSField msgs
    <*> areq dayField (bfs ("Student admission date" :: Text )) (servantGStudentAdmS <$> msgs)
    <*> areq textField (bfs ("Course"::Text)) (servantGStudentCourse <$> msgs)
    <*> areq doubleField (bfs ("GPA"::Text)) (servantGStudentGpa <$> msgs)
    <*> areq textField (bfs ("Graduation"::Text)) (servantGStudentGrad <$> msgs)

    where
        maybeSGSRegEField Nothing = regField
        maybeSGSRegEField (Just tgs) = pure $ servantGStudentRegE tgs

        maybeSGSRegSField Nothing = regField
        maybeSGSRegSField (Just tgs) = pure $ servantGStudentRegS tgs

servantGStudentForm :: Maybe ServantGStudent -> Html -> MForm Handler (FormResult ServantGStudent, Widget)
servantGStudentForm msgs = renderBootstrap3 largeLayout $ servantGStudentAForm msgs