module Main where

import System.IO
import Control.Monad (liftM)
import Control.Exception

import Lib (splitStrings, readMaybeInt)

import UniversityMember

import Student (s_reg)
import Employee (e_reg)


getLines = liftM lines . readFile

main :: IO ()
main = do
    master <- try (getLines "static/mestre.txt") :: IO (Either IOException [String])
    actual <- try (getLines "static/atual.txt") :: IO (Either IOException [String])
    spec <- try (getLines "static/espec.txt") :: IO (Either IOException [String])

    case (master, actual, spec) of
        (Right master', Right actual', Right spec') -> do

            -- parse members
            members <- return $ parseMembers $ splitStrings master' []
            -- update the members list
            actualMembers <- return $ updateMembers (splitStrings actual' [] ) members

            -- get the indexes

            -- get the Teachers sublist sorted asc by registraton
            teachers <- return $ qsortByRule True sortTeacherByReg $ selectMembers isTeacher40 actualMembers

            -- get the undergraduate students sublist sorted desc by age and registraton
            graduatedStudents <- return $ qsortByRule False sortGraduateByAgeReg $ selectMembers isGraduate actualMembers

            -- get the students sublist sorted desc by gpa and registraton
            students <- return $ qsortByRule False sortStudentByGpaReg $ selectMembers isStudent actualMembers

            -- get the Employees sublist sorted desc by salary and registraton
            employees <- return $ qsortByRule False sortEmployeeBySalReg $ selectMembers isEmployee actualMembers

            -- sort all members asc by nallMembers!!me and registraton
            allMembers <- return $ qsortByRule True sortAllByNameReg actualMembers

            case map readMaybeInt spec' of
                [Just i, Just j, Just k, Just l, Just m] -> do

                    tSum <- return $ if i < length teachers && 0 < i
                                        then
                                            e_reg $ teachers!!(i-1)
                                        else
                                            0

                    gSum <- return $ if j < length graduatedStudents && 0 < j
                                        then
                                            s_reg $ graduatedStudents!!(j-1)
                                        else
                                            0
                    sSum <- return $ if k < length students && 0 < k
                                        then
                                            s_reg $ students!!(k-1)
                                        else
                                            0
                    eSum <- return $ if l < length employees && 0 < l
                                        then
                                            e_reg $ employees!!(l-1)
                                        else
                                            0
                    aSum <- return $ if m < length allMembers && 0 < m
                                        then
                                            if hasTwoRegistrations $ allMembers!!(m-1)
                                                then
                                                    getMaxReg (allMembers!!(m-1))
                                                else
                                                    if isEmployee $ allMembers!!(m-1)
                                                        then
                                                            e_reg $ allMembers!!(m-1)
                                                        else
                                                            s_reg $ allMembers!!(m-1)
                                        else
                                            0
                    putStrLn $ show $ tSum + gSum + sSum + eSum + aSum
                _ -> putStrLn "Error! Could not parse the spec.txt file!"

        (Left mstEx, _, _) -> putStrLn $ "Error! Could not open the mestre.txt file: " ++ show mstEx
        (_, Left actEx, _) -> putStrLn $ "Error! Could not open the atual.txt file: " ++ show actEx
        (_, _, Left specEx) -> putStrLn $ "Error! Could not open the espec.txt file: " ++ show specEx



