module ServantGStudent (ServantGStudent (..), readMaybeServantGS) where

import Lib (readMaybeInt, readMaybeInteger, readMaybeFloat)

import Date (Month (..), Date (..), readMaybeDate)
import Person (Genre (..), Person (..), readMaybeGenre)
import Employee (Employee (..), Outsourcing (..), readMaybeOutsourcing)
import Student (Student (..))

data ServantGStudent =
    ServantGStudent {
            sgs_name :: String
        ,   sgs_age :: Int
        ,   sgs_genre :: Genre
        ,   sgs_reg_e :: Integer
        ,   sgs_adm_e :: Date
        ,   sgs_salary :: Float
        ,   sgs_outsourced :: Outsourcing
        ,   sgs_reg_s :: Integer
        ,   sgs_adm_s :: Date
        ,   sgs_course :: String
        ,   sgs_gpa :: Float
        ,   sgs_grad :: String

    } deriving (Show, Eq)

instance Person ServantGStudent where
    name = sgs_name
    age = sgs_age
    genre = sgs_genre

instance Employee ServantGStudent where
    e_reg = sgs_reg_e
    e_adm = sgs_adm_e
    salary = sgs_salary

instance Student ServantGStudent where
    s_reg = sgs_reg_s
    s_adm = sgs_adm_s
    course = sgs_course
    gpa = sgs_gpa

-- read a [String] and try to convert to ServantGStudent
readMaybeServantGS :: [String] -> Maybe ServantGStudent
readMaybeServantGS [csNam, csAge, csGen, csRegE, csAdmE, csSal, csOut, csRegS, csAdmS, csCour, csGpa, csGrad] =
    case (  readMaybeInt csAge,
            readMaybeGenre csGen,
            readMaybeInteger csRegE,
            readMaybeDate csAdmE,
            readMaybeFloat csSal,
            readMaybeOutsourcing csOut,
            readMaybeInteger csRegS,
            readMaybeDate csAdmS,
            readMaybeFloat csGpa ) of
        (   
            Just age',
            Just genre',
            Just regE',
            Just admE',
            Just sal',
            Just out',
            Just regS',
            Just admS',
            Just gpa') -> Just $ ServantGStudent csNam age' genre' regE' admE' sal' out' regS' admS' csCour gpa' csGrad
        _ -> Nothing
readMaybeServantGS _ = Nothing