module ServantUGStudent (ServantUGStudent (..), readMaybeServantUGS) where

import Lib (readMaybeInt, readMaybeInteger, readMaybeFloat)

import Date (Month (..), Date (..), readMaybeDate)
import Person (Genre (..), Person (..), readMaybeGenre)
import Employee (Employee (..), Outsourcing (..), readMaybeOutsourcing)
import Student (Student (..))

data ServantUGStudent =
    ServantUGStudent {
            sugs_name :: String
        ,   sugs_age :: Int
        ,   sugs_genre :: Genre
        ,   sugs_reg_e :: Integer
        ,   sugs_adm_e :: Date
        ,   sugs_salary :: Float
        ,   sugs_outsourced :: Outsourcing
        ,   sugs_reg_s :: Integer
        ,   sugs_adm_s :: Date
        ,   sugs_course :: String
        ,   sugs_gpa :: Float
        ,   sugs_score :: Int

    } deriving (Show, Eq)

instance Person ServantUGStudent where
    name = sugs_name
    age = sugs_age
    genre = sugs_genre

instance Employee ServantUGStudent where
    e_reg = sugs_reg_e
    e_adm = sugs_adm_e
    salary = sugs_salary

instance Student ServantUGStudent where
    s_reg = sugs_reg_s
    s_adm = sugs_adm_s
    course = sugs_course
    gpa = sugs_gpa

-- read a [String] and try to convert to ServantUGStudent
readMaybeServantUGS :: [String] -> Maybe ServantUGStudent
readMaybeServantUGS [csNam, csAge, csGen, csRegE, csAdmE, csSal, csOut, csRegS, csAdmS, csCour, csGpa, csSco] =
    case (  readMaybeInt csAge,
            readMaybeGenre csGen,
            readMaybeInteger csRegE,
            readMaybeDate csAdmE,
            readMaybeFloat csSal,
            readMaybeOutsourcing csOut,
            readMaybeInteger csRegS,
            readMaybeDate csAdmS,
            readMaybeFloat csGpa,
            readMaybeInt csSco ) of
        (   
            Just age',
            Just genre',
            Just regE',
            Just admE',
            Just sal',
            Just out',
            Just regS',
            Just admS',
            Just gpa',
            Just sco') -> Just $ ServantUGStudent csNam age' genre' regE' admE' sal' out' regS' admS' csCour gpa' sco'
        _ -> Nothing
readMaybeServantUGS _ = Nothing