module TeacherGStudent (TeacherGStudent (..), readMaybeTeacherGStudent) where

import Lib (readMaybeInt, readMaybeInteger, readMaybeFloat)

import Date (Month (..), Date (..), readMaybeDate)
import Person (Genre (..), Person (..), readMaybeGenre)
import Student (Student (..))
import Employee (Employee (..), Regime (..), readMaybeRegime)

data TeacherGStudent = 
    TeacherGStudent {
            tgs_name :: String
        ,   tgs_age :: Int
        ,   tgs_genre :: Genre
        ,   tgs_reg_e :: Integer
        ,   tgs_adm_e :: Date
        ,   tgs_salary :: Float
        ,   tgs_regime :: Regime
        ,   tgs_reg_s :: Integer
        ,   tgs_adm_s :: Date
        ,   tgs_course :: String
        ,   tgs_gpa :: Float
        ,   tgs_grad :: String
    } deriving (Show, Eq)

-- the Person class implementation
instance Person TeacherGStudent where
    name = tgs_name
    genre = tgs_genre
    age = tgs_age

-- the Employee class implementation
instance Employee TeacherGStudent where
    e_reg = tgs_reg_e
    e_adm = tgs_adm_e
    salary = tgs_salary

-- the Student class implementation
instance Student TeacherGStudent where
    s_reg = tgs_reg_s
    s_adm = tgs_adm_s
    course = tgs_course
    gpa = tgs_gpa

-- read a [String] and try to convert to TeacherGStudent
readMaybeTeacherGStudent :: [String] -> Maybe TeacherGStudent
readMaybeTeacherGStudent   [ tNam, tAge, tGen, tRgE, tAdmE, tSal, tRegime, tRgS, tAdmS, tCourse, tGpa, tGrad ] =
    case (  readMaybeInt tAge,
            readMaybeGenre tGen,
            readMaybeInteger tRgE,
            readMaybeDate tAdmE,
            readMaybeFloat tSal,
            readMaybeRegime tRegime,
            readMaybeInteger tRgS,
            readMaybeDate tAdmS,
            readMaybeFloat tGpa
        ) of
        (
            Just age',
            Just genre',
            Just regE',
            Just admE',
            Just sal',
            Just regime',
            Just regS',
            Just admS',
            Just gpa' ) -> Just $ TeacherGStudent tNam age' genre' regE' admE' sal' regime' regS' admS' tCourse gpa' tGrad 
        _ -> Nothing
readMaybeTeacherGStudent _ = Nothing