module TeacherUGStudent (TeacherUGStudent (..), readMaybeTeacherUGStudent) where

import Lib (readMaybeInt, readMaybeInteger, readMaybeFloat)

import Date (Month (..), Date (..), readMaybeDate)
import Person (Genre (..), Person (..), readMaybeGenre)
import Student (Student (..))
import Employee (Employee (..), Regime (..), readMaybeRegime)

data TeacherUGStudent = 
    TeacherUGStudent {
            tugs_name :: String
       ,    tugs_age :: Int
       ,    tugs_genre :: Genre
       ,    tugs_reg_e :: Integer
       ,    tugs_adm_e :: Date
       ,    tugs_salary :: Float
       ,    tugs_regime :: Regime
       ,    tugs_reg_s :: Integer
       ,    tugs_adm_s :: Date
       ,    tugs_course :: String
       ,    tugs_gpa :: Float
       ,    tugs_score :: Int
    } deriving (Show, Eq)


-- the Person class implementation
instance Person TeacherUGStudent where
    name = tugs_name
    genre = tugs_genre
    age = tugs_age

-- the Employee class implementation
instance Employee TeacherUGStudent where
    e_reg = tugs_reg_e
    e_adm = tugs_adm_e
    salary = tugs_salary

-- the Student class implementation
instance Student TeacherUGStudent where
    s_reg = tugs_reg_s
    s_adm = tugs_adm_s
    course = tugs_course
    gpa = tugs_gpa

-- read a [String] and try to convert to TeacherUGStudent
readMaybeTeacherUGStudent :: [String] -> Maybe TeacherUGStudent
readMaybeTeacherUGStudent   [ tNam, tAge, tGen, tRgE, tAdmE, tSal, tRegime, tRgS, tAdmS, tCourse, tGpa, tSco ] =
    case (  readMaybeInt tAge,
            readMaybeGenre tGen,
            readMaybeInteger tRgE,
            readMaybeDate tAdmE,
            readMaybeFloat tSal,
            readMaybeRegime tRegime,
            readMaybeInteger tRgS,
            readMaybeDate tAdmS,
            readMaybeFloat tGpa,
            readMaybeInt tSco ) of
        (
            Just age',
            Just genre',
            Just regE',
            Just admE',
            Just sal',
            Just regime',
            Just regS',
            Just admS',
            Just gpa',
            Just score') -> Just $ TeacherUGStudent tNam age' genre' regE' admE' sal' regime' regS' admS' tCourse gpa' score' 
        _ -> Nothing
readMaybeTeacherUGStudent _ = Nothing