module UniversityMember (
        UniversityMember (..)
    ,   selectMembers
    ,   isTeacher
    ,   isTeacher20
    ,   isTeacher40
    ,   isTeacherED
    ,   isStudent
    ,   isUndergraduate
    ,   isGraduate
    ,   isEmployee
    ,   hasTwoRegistrations
    ,   getMaxReg
    ,   sortAllByNameReg
    ,   sortStudentByGpaReg
    ,   sortTeacherByReg
    ,   sortEmployeeBySalReg
    ,   sortGraduateByAgeReg
    ,   qsortByRule
    ,   parseMembers
    ,   updateMembers
    ,   buildMember
    ,   editMember
    ,   removeMemberByReg
    ,   removeEmployeeData
    ,   removeStudentData
    ,   showMembers
    ) where

import Data.List (intersect)

import Date (Month (..), Date (..))
import Person (Genre (..), Person (..))
import Employee (Regime (..), Employee (..))
import Student (Student (..))

import UGStudent (UGStudent (..), readMaybeUGStudent)
import GStudent (GStudent (..), readMaybeGStudent)
import Teacher (Teacher (..), readMaybeTeacher)
import TeacherUGStudent (TeacherUGStudent (..), readMaybeTeacherUGStudent)
import TeacherGStudent (TeacherGStudent (..), readMaybeTeacherGStudent)
import CivilServant (CivilServant (..), readMaybeCivilServant)
import ServantUGStudent (ServantUGStudent (..), readMaybeServantUGS)
import ServantGStudent (ServantGStudent (..), readMaybeServantGS)

import Lib (readMaybeInteger)

data UniversityMember =
        UGSMember UGStudent
    |   GSMember GStudent
    |   TMember Teacher
    |   TUGSMember TeacherUGStudent
    |   TGSMember TeacherGStudent
    |   CSMember CivilServant
    |   SUGSMember ServantUGStudent
    |   SGSMember ServantGStudent

    deriving (Show, Eq)

instance Person UniversityMember where
    name member =
        case member of
            UGSMember m -> name m
            GSMember m -> name m
            TMember m -> name m
            TUGSMember m -> name m
            TGSMember m -> name m
            CSMember m -> name m
            SUGSMember m -> name m
            SGSMember m -> name m
    genre member =
        case member of
            UGSMember m -> genre m
            GSMember m  -> genre m
            TMember m -> genre m
            TUGSMember m -> genre m
            TGSMember m -> genre m
            CSMember m -> genre m
            SUGSMember m -> genre m
            SGSMember m ->  genre m
    age member =
        case member of
            UGSMember m -> age m
            GSMember m -> age m
            TMember m -> age m
            TUGSMember m -> age m
            TGSMember m -> age m
            CSMember m -> age m
            SUGSMember m -> age m
            SGSMember m -> age m

instance Employee UniversityMember where
    e_reg member =
        case member of
            TMember m -> e_reg m
            TUGSMember m -> e_reg m
            TGSMember m -> e_reg m
            CSMember m -> e_reg m
            SUGSMember m -> e_reg m
            SGSMember m -> e_reg m
    e_adm member =
        case member of
            TMember m -> e_adm m
            TUGSMember m -> e_adm m
            TGSMember m -> e_adm m
            CSMember m -> e_adm m
            SUGSMember m -> e_adm m
            SGSMember m -> e_adm m
    salary member =
        case member of
            TMember m -> salary m
            TUGSMember m -> salary m
            TGSMember m -> salary m
            CSMember m -> salary m
            SUGSMember m -> salary m
            SGSMember m -> salary m

instance Student UniversityMember where
    s_reg member =
        case member of
            UGSMember m -> s_reg m
            GSMember m -> s_reg m
            TUGSMember m -> s_reg m
            TGSMember m -> s_reg m
            SUGSMember m -> s_reg m
            SGSMember m -> s_reg m

    s_adm member =
        case member of
            UGSMember m -> s_adm m
            GSMember m -> s_adm m
            TUGSMember m -> s_adm m
            TGSMember m -> s_adm m
            SUGSMember m -> s_adm m
            SGSMember m -> s_adm m

    course member =
        case member of
            UGSMember m -> course m
            GSMember m -> course m
            TUGSMember m -> course m
            TGSMember m -> course m
            SUGSMember m -> course m
            SGSMember m -> course m

    gpa member =
        case member of
            UGSMember m -> gpa m
            GSMember m -> gpa m
            TUGSMember m -> gpa m
            TGSMember m -> gpa m
            SUGSMember m -> gpa m
            SGSMember m -> gpa m

---------------------------------------------------------------------------
---------------------------------------------------------------------------
-- get all Students from an UniversityMember list
selectMembers :: (UniversityMember -> Bool) -> [UniversityMember] -> [UniversityMember]
selectMembers _ [] = []
selectMembers select ms = [x | x <- ms, select x]

---------------------------------------------------------------------------
---------------------------------------------------------------------------
-- the selection methods
isTeacher :: UniversityMember -> Bool
isTeacher member =
    case member of
        TMember _ -> True
        TUGSMember _ -> True
        TGSMember _ -> True
        _ -> False

-- the selection methods
isTeacher40 :: UniversityMember -> Bool
isTeacher40 member =
    case member of
        TMember t -> R40 == t_regime t
        TUGSMember t -> R40 == tugs_regime t
        TGSMember t -> R40 == tgs_regime t
        _ -> False

-- the selection methods
isTeacher20 :: UniversityMember -> Bool
isTeacher20 member =
    case member of
        TMember t -> R20 == t_regime t
        TUGSMember t -> R20 == tugs_regime t
        TGSMember t -> R20 == tgs_regime t
        _ -> False

-- the selection methods
isTeacherED :: UniversityMember -> Bool
isTeacherED member =
    case member of
        TMember t -> RED == t_regime t
        TUGSMember t -> RED == tugs_regime t
        TGSMember t -> RED == tgs_regime t
        _ -> False
---------------------------------------------------------------------------
---------------------------------------------------------------------------
-- the students selection methods
-- verify if an UniversityMember is a Student
isStudent :: UniversityMember -> Bool
isStudent member =
    case member of
        TMember _ -> False
        CSMember _ -> False
        _ -> True

-- verify if an UniversityMember is an Undergraduate Student
isUndergraduate :: UniversityMember -> Bool
isUndergraduate member =
    case member of
        UGSMember _ -> True
        TUGSMember _ -> True
        SUGSMember _ -> True
        _ -> False

-- verify if an UniversityMember is an Graduate Student
isGraduate :: UniversityMember -> Bool
isGraduate member =
    case member of
        GSMember _ -> True
        TGSMember _ -> True
        SGSMember _ -> True
        _ -> False

-- verify if an UniversityMember has 2 registrations
hasTwoRegistrations :: UniversityMember -> Bool
hasTwoRegistrations member =
    case member of
        UGSMember _ -> False
        GSMember _ -> False
        TMember _ -> False
        CSMember _ -> False
        _ -> True

---------------------------------------------------------------------------
---------------------------------------------------------------------------
-- the Employees selection methods
-- verify if an UniversityMember is an Employee
isEmployee :: UniversityMember -> Bool
isEmployee member =
    case member of
        UGSMember _ -> False
        GSMember _ -> False
        _ -> True
---------------------------------------------------------------------------
---------------------------------------------------------------------------
-- the Civil Servants selection methods
-- verify if an UniversityMember is a Civil Servant
isCivilServant :: UniversityMember -> Bool
isCivilServant member =
    case member of
        CSMember _ -> True
        SUGSMember _ -> True
        SGSMember _ -> True
        _ -> False

------------------------------------------------------------------------------
------------------------------------------------------------------------------
getMaxReg :: UniversityMember -> Integer
getMaxReg member =
    case member of
        UGSMember m -> s_reg m
        GSMember m -> s_reg m
        TMember m -> e_reg m
        TUGSMember m -> max (e_reg m) (s_reg m)
        TGSMember m -> max (e_reg m) (s_reg m)
        CSMember m -> e_reg m
        SUGSMember m -> max (e_reg m) (s_reg m)
        SGSMember m -> max (e_reg m) (s_reg m)

------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- a rule to ord UniversityMember by name and registration
sortAllByNameReg :: UniversityMember -> UniversityMember -> Bool
sortAllByNameReg a b = name a < name b || name a == name b && getMaxReg a < getMaxReg b

-- a rule to sort Teachers with 40 h regime by employee registration
sortTeacherByReg :: UniversityMember -> UniversityMember -> Bool
sortTeacherByReg a b = e_reg a < e_reg b

-- a rule to sort Graduate Students by age
sortGraduateByAgeReg :: UniversityMember -> UniversityMember -> Bool
sortGraduateByAgeReg a b = age a < age b || age a == age b && s_reg a < s_reg b 

-- a rule to sort all Students by gpa and registration
sortStudentByGpaReg :: UniversityMember -> UniversityMember -> Bool
sortStudentByGpaReg a b = gpa a < gpa b || gpa a == gpa b && s_reg a < s_reg b

-- a rule to sort all Employees by salary and registration
sortEmployeeBySalReg :: UniversityMember -> UniversityMember -> Bool
sortEmployeeBySalReg a b = salary a < salary b || salary a == salary b && e_reg a < e_reg b

---------------------------------------------------------------------------
---------------------------------------------------------------------------
qsortByRuleAsc :: (UniversityMember -> UniversityMember -> Bool) -> [UniversityMember] -> [UniversityMember]
qsortByRuleAsc _ [] = []
qsortByRuleAsc rule (m:ms) =
    qsortByRuleAsc rule leftSide ++ [m] ++ qsortByRuleAsc rule rightSide
    where
        leftSide =  [x | x <- ms, rule x m]
        rightSide = [x | x <- ms, not $ rule x m]

qsortByRuleDesc :: (UniversityMember -> UniversityMember -> Bool) -> [UniversityMember] -> [UniversityMember]
qsortByRuleDesc _ [] = []
qsortByRuleDesc rule (m:ms) =
    qsortByRuleDesc rule leftSide ++ [m] ++ qsortByRuleDesc rule rightSide
    where
        leftSide =  [x | x <- ms, not $ rule x m]
        rightSide = [x | x <- ms, rule x m]

qsortByRule :: Bool -> (UniversityMember -> UniversityMember -> Bool) -> [UniversityMember] -> [UniversityMember]
qsortByRule True rule ms = qsortByRuleAsc rule ms
qsortByRule False rule ms = qsortByRuleDesc rule ms

---------------------------------------------------------------------------
---------------------------------------------------------------------------
parseMembers :: [[String]] -> [UniversityMember]
parseMembers [] = []
parseMembers (m:ms) = 
    case buildMember m of
        Just member -> member : parseMembers ms
        Nothing -> parseMembers ms

---------------------------------------------------------------------------
---------------------------------------------------------------------------

buildMember :: [String] -> Maybe UniversityMember
buildMember [] = Nothing
buildMember (mt:md) = 
    case mt of
        "grad" ->       case readMaybeUGStudent md of
                            Just m -> Just $ UGSMember m
                            _ -> Nothing
        "pos" ->        case readMaybeGStudent md of
                            Just m -> Just $ GSMember m
                            _ -> Nothing
        "prof" ->       case readMaybeTeacher md of
                            Just m -> Just $ TMember m
                            _ -> Nothing
        "prof_grad" ->  case readMaybeTeacherUGStudent md of
                            Just m -> Just $ TUGSMember m
                            _ -> Nothing
        "prof_pos" ->   case readMaybeTeacherGStudent md of
                            Just m -> Just $ TGSMember m
                            _ -> Nothing
        "serv" ->       case readMaybeCivilServant md of
                            Just m -> Just $ CSMember m
                            _ -> Nothing
        "serv_grad" ->  case readMaybeServantUGS md of
                            Just m -> Just $ SUGSMember m
                            _ -> Nothing
        "serv_pos" ->   case readMaybeServantGS md of
                            Just m -> Just $ SGSMember m
                            _ -> Nothing
        _ -> Nothing
---------------------------------------------------------------------------
---------------------------------------------------------------------------
updateMembers :: [[String]] -> [UniversityMember] -> [UniversityMember]
updateMembers _ [] = []
updateMembers [] ms = ms
updateMembers ((c:d):cmds) ms =
    case c of
        "i" -> case buildMember d of
                    Just member -> updateMembers cmds (member : ms)
                    _ -> updateMembers cmds ms
        "e" -> case readMaybeInteger $ head d of
                Just reg' -> updateMembers cmds (removeMemberByReg reg' ms)
                Nothing -> updateMembers cmds ms
        "a" -> case buildMember d of
                    Just member -> updateMembers cmds (editMember member ms)
                    Nothing -> updateMembers cmds ms
        _ -> updateMembers cmds ms

-- remove a single member from the UniversityMember list
removeMemberByReg :: Integer -> [UniversityMember] ->  [UniversityMember]
removeMemberByReg _ [] = []
removeMemberByReg reg' (um:ms) =
    if hasTwoRegistrations um
        then
            -- the member has 2 registrations
            if e_reg um == reg'
                then
                    (removeEmployeeData um) : ms
                else
                    if s_reg um == reg'
                        then
                            removeStudentData um : ms
                        else
                            um : removeMemberByReg reg' ms
        else
            -- the member has only 1 registration
            if isStudent um
                then
                    if s_reg um == reg'
                        then
                            ms
                        else
                            um : removeMemberByReg reg' ms
                else
                    -- it's an employee
                    if e_reg um == reg'
                        then
                            ms
                        else
                            um : removeMemberByReg reg' ms


-- remove employee data
removeEmployeeData :: UniversityMember -> UniversityMember
removeEmployeeData (TUGSMember (TeacherUGStudent tName tAge tGen _ _ _ _ tRegS tAdmS tCour tGpa tSco)) =
    UGSMember $ UGStudent tName tAge tGen tRegS tAdmS tCour tGpa tSco
removeEmployeeData (TGSMember (TeacherGStudent tName tAge tGen _ _ _ _ tRegS tAdmS tCour tGpa tGrad)) =
    GSMember $ GStudent tName tAge tGen tRegS tAdmS tCour tGpa tGrad
removeEmployeeData (SUGSMember (ServantUGStudent csNam csAge csGen _ _ _ _ csRegS csAdmS csCour csGpa csSco)) =
    UGSMember $ UGStudent csNam csAge csGen csRegS csAdmS csCour csGpa csSco
removeEmployeeData (SGSMember (ServantGStudent csNam csAge csGen _ _ _ _ csRegS csAdmS csCour csGpa csGrad)) =
    GSMember $ GStudent csNam csAge csGen csRegS csAdmS csCour csGpa csGrad
removeEmployeeData um = um

-- remove student data
removeStudentData :: UniversityMember -> UniversityMember
removeStudentData (TUGSMember (TeacherUGStudent tName tAge tGen tRegE tAdmE tSal tRegime _ _ _ _ _)) =
    TMember $ Teacher tName tAge tGen tRegE tAdmE tSal tRegime
removeStudentData (TGSMember (TeacherGStudent tName tAge tGen tRegE tAdmE tSal tRegime _ _ _ _ _)) =
    TMember $ Teacher tName tAge tGen tRegE tAdmE tSal tRegime
removeStudentData (SUGSMember (ServantUGStudent csNam csAge csGen csRegE csAdmE csSal csOut _ _ _ _ _)) =
    CSMember $ CivilServant csNam csAge csGen csRegE csAdmE csSal csOut
removeStudentData (SGSMember (ServantGStudent csNam csAge csGen csRegE csAdmE csSal csOut _ _ _ _ _)) =
    CSMember $ CivilServant csNam csAge csGen csRegE csAdmE csSal csOut
removeStudentData um = um

-- edit an UniversityMember with new data
editMember :: UniversityMember -> [UniversityMember] -> [UniversityMember]
editMember _ [] = []
editMember member (m:ms) = 
    if sameReg member m
        then
            member : ms
        else
            m : editMember member ms

---------------------------------------------------------------------------
---------------------------------------------------------------------------
-- verify if the registration are the same
sameReg :: UniversityMember -> UniversityMember -> Bool
sameReg a b = 
    not $ null $ intersect regA regB
    where
        regA = [fstReg a, sndReg a]
        regB = [fstReg b, sndReg b]


fstReg :: UniversityMember -> Integer
fstReg member =
    case member of
        UGSMember m -> s_reg m
        GSMember m -> s_reg m
        _ -> e_reg member

sndReg :: UniversityMember -> Integer
sndReg member =
    case member of
        TMember m -> e_reg m
        CSMember m -> e_reg m
        _ -> s_reg member

---------------------------------------------------------------------------
---------------------------------------------------------------------------
-- print the Members
showMembers :: [UniversityMember] -> IO ()
showMembers [] = return ()
showMembers (m:ms) = do
    putStrLn $ show m
    showMembers ms
